@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014 Cisco and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -60,7 +60,7 @@ static struct dconf_module modules[] = {
     { "PE",     "MD5SECT",      PE_CONF_MD5SECT,        1 },
     { "PE",     "UPX",      PE_CONF_UPX,        1 },
     { "PE",     "FSG",      PE_CONF_FSG,        1 },
-    { "PE",     "SWIZZOR",      PE_CONF_SWIZZOR,        1 },
+    { "PE",     "SWIZZOR",      PE_CONF_SWIZZOR,        0 },
 
     { "PE",     "PETITE",       PE_CONF_PETITE,     1 },
     { "PE",     "PESPIN",       PE_CONF_PESPIN,     1 },
@@ -72,9 +72,8 @@ static struct dconf_module modules[] = {
     { "PE",     "UPACK",        PE_CONF_UPACK,      1 },
     { "PE",     "ASPACK",       PE_CONF_ASPACK,     1 },
     { "PE",     "CATALOG",      PE_CONF_CATALOG,        1 },
-    { "PE",     "DISABLECERT",  PE_CONF_DISABLECERT,    0 },
-    { "PE",     "DUMPCERT",     PE_CONF_DUMPCERT,       0 },
-    { "PE",     "MATCHICON",     PE_CONF_MATCHICON,       1 },
+    { "PE",     "CERTS",        PE_CONF_CERTS,      1 },
+    { "PE",     "MATCHICON",    PE_CONF_MATCHICON,       1 },
 
     { "ELF",        NULL,       0x1,            1 },
 
@@ -102,6 +101,10 @@ static struct dconf_module modules[] = {
     { "ARCHIVE",    "XAR",      ARCH_CONF_XAR,      1 },
     { "ARCHIVE",    "HFSPLUS",      ARCH_CONF_HFSPLUS,      1 },
     { "ARCHIVE",    "XZ",       ARCH_CONF_XZ,       1 },
+    { "ARCHIVE",    "PASSWD",   ARCH_CONF_PASSWD,   1 },
+    { "ARCHIVE",    "MBR",      ARCH_CONF_MBR,      1 },
+    { "ARCHIVE",    "GPT",      ARCH_CONF_GPT,      1 },
+    { "ARCHIVE",    "APM",      ARCH_CONF_APM,      1 },
 
     { "DOCUMENT",   "HTML",     DOC_CONF_HTML,      1 },
     { "DOCUMENT",   "RTF",      DOC_CONF_RTF,       1 },
@@ -110,6 +113,9 @@ static struct dconf_module modules[] = {
     { "DOCUMENT",   "HTMLSKIPRAW",  DOC_CONF_HTML_SKIPRAW,  1 },
     { "DOCUMENT",   "JSNORM",       DOC_CONF_JSNORM,        1 },
     { "DOCUMENT",   "SWF",      DOC_CONF_SWF,       1 },
+    { "DOCUMENT",   "OOXML",    DOC_CONF_OOXML,     1 },
+    { "DOCUMENT",   "MSPML",    DOC_CONF_MSXML,     1 },
+    { "DOCUMENT",   "HWP",      DOC_CONF_HWP,       1 },
 
     { "MAIL",       "MBOX",     MAIL_CONF_MBOX,     1 },
     { "MAIL",       "TNEF",     MAIL_CONF_TNEF,     1 },
@@ -136,6 +142,10 @@ static struct dconf_module modules[] = {
     { "STATS",      "DISABLED",     DCONF_STATS_DISABLED,   0 },
     { "STATS",      "PESECTION DISABLED", DCONF_STATS_PE_SECTION_DISABLED, 0 },
 
+    { "PCRE",       "SUPPORT",      PCRE_CONF_SUPPORT,   1 },
+    { "PCRE",       "OPTIONS",      PCRE_CONF_OPTIONS,   1 },
+    { "PCRE",       "GLOBAL",       PCRE_CONF_GLOBAL,    1 },
+
     { NULL,     NULL,       0,              0 }
 };
 
@@ -189,6 +199,9 @@ struct cli_dconf *cli_dconf_init(void)
         } else if (!strcmp(modules[i].mname, "STATS")) {
             if (modules[i].state)
                 dconf->stats |= modules[i].bflag;
+        } else if (!strcmp(modules[i].mname, "PCRE")) {
+            if (modules[i].state)
+                dconf->pcre |= modules[i].bflag;
         }
     }
 
@@ -198,7 +211,7 @@ struct cli_dconf *cli_dconf_init(void)
 void cli_dconf_print(struct cli_dconf *dconf)
 {
     unsigned int pe = 0, elf = 0, macho = 0, arch = 0, doc = 0, mail = 0;
-    unsigned int other = 0, phishing = 0, i, bytecode=0, stats=0;
+    unsigned int other = 0, phishing = 0, i, bytecode=0, stats=0, pcre=0;
 
 
     cli_dbgmsg("Dynamic engine configuration settings:\n");
@@ -292,9 +305,28 @@ void cli_dconf_print(struct cli_dconf *dconf)
             }
 
             if (dconf->stats)
-                cli_dbgmsg("    * Submodule %10s:\t%s\n", modules[i].sname, (dconf->stats & modules[i].bflag) ? "On" : "** Off **");
+                cli_dbgmsg("   * Submodule %10s:\t%s\n", modules[i].sname, (dconf->stats & modules[i].bflag) ? "On" : "** Off **");
             else
                 continue;
+        } else if (!strcmp(modules[i].mname, "PCRE")) {
+#if HAVE_PCRE
+            if (!pcre) {
+                cli_dbgmsg("Module PCRE %s\n", dconf->pcre ? "On" : "Off");
+                pcre = 1;
+            }
+
+            if (dconf->pcre)
+                cli_dbgmsg("   * Submodule %10s:\t%s\n", modules[i].sname, (dconf->pcre & modules[i].bflag) ? "On" : "** Off **");
+            else
+                continue;
+#else
+            if (!pcre) {
+                cli_dbgmsg("Module PCRE Off\n");
+                pcre = 1;
+            }
+
+            continue;
+#endif
         }
     }
 }
@@ -366,8 +398,8 @@ int cli_dconf_load(FILE *fs, struct cl_engine *engine, unsigned int options, str
             }
         }
 
-        if(!strncmp(buffer, "MACHO:", 4) && chkflevel(buffer, 2)) {
-            if(sscanf(buffer + 4, "0x%x", &val) == 1) {
+        if(!strncmp(buffer, "MACHO:", 6) && chkflevel(buffer, 2)) {
+            if(sscanf(buffer + 6, "0x%x", &val) == 1) {
                 engine->dconf->macho = val;
             } else {
                 ret = CL_EMALFDB;
@@ -437,6 +469,15 @@ int cli_dconf_load(FILE *fs, struct cl_engine *engine, unsigned int options, str
                 break;
             }
         }
+
+        if(!strncmp(buffer, "PCRE:", 5) && chkflevel(buffer, 2)) {
+            if(sscanf(buffer + 5, "0x%x", &val) == 1) {
+                engine->dconf->pcre = val;
+            } else {
+                ret = CL_EMALFDB;
+                break;
+            }
+        }
     }
 
     if(ret) {
