@@ -156,17 +156,20 @@ static int msxml_parse_value(json_object *wrkptr, const char *arrname, const xml
 }
 #endif /* HAVE_JSON */
 
+#define MAX_ATTRIBS 20
 static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader, int rlvl, void *jptr)
 {
     const xmlChar *element_name = NULL;
     const xmlChar *node_name = NULL, *node_value = NULL;
     const struct key_entry *keyinfo;
-    int ret, virus = 0, state, node_type, endtag = 0;
+    struct attrib_entry attribs[MAX_ATTRIBS];
+    int ret, virus = 0, state, node_type, endtag = 0, num_attribs = 0;
     cli_ctx *ctx = mxctx->ctx;
 #if HAVE_JSON
     json_object *parent = (json_object *)jptr;
     json_object *thisjobj = NULL;
 #else
+    void *parent = NULL;
     void *thisjobj = NULL;
 #endif
 
@@ -258,6 +261,21 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
                 cli_msxmlmsg("msxml_parse_element: retrieved json object [Count]\n");
             }
 
+            /* check if multiple entries are allowed */
+            if (thisjobj && (keyinfo->type & MSXML_JSON_MULTI)) {
+                /* replace this object with an array entry object */
+                json_object *multi = cli_jsonarray(thisjobj, "Multi");
+                if (!multi) {
+                    return CL_EMEM;
+                }
+                cli_msxmlmsg("msxml_parse_element: generated or retrieved json multi array\n");
+
+                thisjobj = cli_jsonobj(multi, NULL);
+                if (!thisjobj)
+                    return CL_EMEM;
+                cli_msxmlmsg("msxml_parse_element: generated json multi entry object\n");
+            }
+
             /* handle attributes */
             if (thisjobj && (keyinfo->type & MSXML_JSON_ATTRIB)) {
                 state = xmlTextReaderHasAttributes(reader);
@@ -284,6 +302,35 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
             }
         }
 #endif
+        /* populate attributes for scanning callback - BROKEN, probably from the fact the reader is pointed to the attribute from previously parsing attributes */
+        if ((keyinfo->type & MSXML_SCAN_CB) && mxctx->scan_cb) {
+            state = xmlTextReaderHasAttributes(reader);
+            if (state == 0) {
+                state = xmlTextReaderMoveToFirstAttribute(reader);
+                if (state == 1) {
+                    /* read first attribute (current head) */
+                    attribs[num_attribs].key = (const char *)xmlTextReaderConstLocalName(reader);
+                    attribs[num_attribs].value = (const char *)xmlTextReaderConstValue(reader);
+                    num_attribs++;
+                } else if (state == -1) {
+                    return CL_EPARSE;
+                }
+            }
+
+            /* start reading attributes or read remainder of attributes */
+            if (state == 1) {
+                cli_msxmlmsg("msxml_parse_element: adding attributes to scanning context\n");
+
+                while ((num_attribs < MAX_ATTRIBS) && (xmlTextReaderMoveToNextAttribute(reader) == 1)) {
+                    attribs[num_attribs].key = (const char *)xmlTextReaderConstLocalName(reader);
+                    attribs[num_attribs].value = (const char *)xmlTextReaderConstValue(reader);
+                    num_attribs++;
+                }
+            }
+            else if (state == -1) {
+                return CL_EPARSE;
+            }
+        }
 
         /* check self-containment */
         state = xmlTextReaderMoveToElement(reader);
@@ -316,7 +363,7 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
 
             switch (node_type) {
             case XML_READER_TYPE_ELEMENT:
-                ret = msxml_parse_element(mxctx, reader, rlvl+1, thisjobj);
+                ret = msxml_parse_element(mxctx, reader, rlvl+1, thisjobj ? thisjobj : parent);
                 if (ret != CL_SUCCESS || (!SCAN_ALL && ret == CL_VIRUS)) {
                     return ret;
                 } else if (SCAN_ALL && ret == CL_VIRUS) {
@@ -339,8 +386,39 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
                     cli_msxmlmsg("msxml_parse_element: added json value [%s: %s]\n", keyinfo->name, (const char *)node_value);
                 }
 #endif
+                /* callback-based scanning mechanism for embedded objects (used by HWPML) */
+                if ((keyinfo->type & MSXML_SCAN_CB) && mxctx->scan_cb) {
+                    char name[1024];
+                    char *tempfile = name;
+                    int of;
+                    size_t vlen = strlen((const char *)node_value);
 
-                /* scanning protocol for embedded objects encoded in base64 */
+                    cli_msxmlmsg("BINARY CALLBACK DATA!\n");
+
+                    if ((ret = cli_gentempfd(ctx->engine->tmpdir, &tempfile, &of)) != CL_SUCCESS) {
+                        cli_warnmsg("msxml_parse_element: failed to create temporary file %s\n", tempfile);
+                        return ret;
+                    }
+
+                    if (cli_writen(of, (char *)node_value, vlen) != vlen) {
+                        close(of);
+                        return CL_EWRITE;
+                    }
+
+                    cli_dbgmsg("msxml_parse_element: extracted binary data to %s\n", tempfile);
+
+                    ret = mxctx->scan_cb(of, ctx, num_attribs, attribs);
+                    if (!(ctx->engine->keeptmp))
+                        cli_unlink(tempfile);
+                    free(tempfile);
+                    if (ret != CL_SUCCESS && (ret != CL_VIRUS || (!SCAN_ALL && ret == CL_VIRUS))) {
+                        return ret;
+                    } else if (SCAN_ALL && ret == CL_VIRUS) {
+                        virus = 1;
+                    }
+                }
+
+                /* scanning protocol for embedded objects encoded in base64 (used by MSXML) */
                 if (keyinfo->type & MSXML_SCAN_B64) {
                     char name[1024];
                     char *decoded, *tempfile = name;
@@ -445,7 +523,7 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
 }
 
 /* reader intialization and closing handled by caller */
-int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct key_entry *keys, const size_t num_keys, int mode)
+int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct key_entry *keys, const size_t num_keys, int mode, msxml_scan_cb scan_cb)
 {
     struct msxml_ctx mxctx;
     int state, virus = 0, ret = CL_SUCCESS;
@@ -454,6 +532,7 @@ int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct
         return CL_ENULLARG;
 
     mxctx.ctx = ctx;
+    mxctx.scan_cb = scan_cb;
     mxctx.keys = keys;
     mxctx.num_keys = num_keys;
 #if HAVE_JSON
